function varargout=bimoc_simulate(what,varargin);
% Implementation of Bimanual optimal control (BIMOC) simulation for the
% paper "Optimal task dependent changes of feedback control and
% adaptation" (Diedrichsen, 200X, Current Biology).

figtitle={'one cursor','two cusor'};
switch (what)
    case 'simulate_forcefield';
        cursor=varargin{1};
        M=bimoc_system('system');
        J=bimoc_system('costfunction',M,'cursor',cursor);
        M1=bimoc_system('forcefield',M,4.5);
        M2=bimoc_system('forcefield',M,-4.5);
        V=oc_solveLQG(M,J);
        [x1,u1]=oc_runsystem(M,V,J,'plant',M);
        [x2,u2]=oc_runsystem(M,V,J,'plant',M1);
        [x3,u3]=oc_runsystem(M,V,J,'plant',M2);
        varargout={x1,x2,x3,M};
    
        % ---------------------------------------------------
        % Generate plot of experimental data - supplementary Figure 1
        % The data structure is averaged across hands and subject already
        % D.cursor: cursor condition
        % D.perturbS: Perturbation on own hand
        % D.perturbO: Perturbation on other hand
        % T.t: Time (ms)
        % T.x: average x-coordinate
        % T.y: average y-coordinate
        % T.vx: average x-velocity
        % T.vy: average y-velocity
    case 'data_plot'
        load trace_summary.mat;
        format={'b','k','r','b','r'};
        for cursor=1:2

            % Plot x-y plot of the perturbed and unperturbed hand
            subplot(4,2,cursor);
            for p=[-1 0 1]
                indx=find(D.cursor==cursor & D.perturbS==p & D.perturbO==0);
                plot(D.x(indx,:)/10-2,D.y(indx,:)/10,[format{p+2} '.']);hold on;
            end;
            for p=[-1 0 1]
                indx=find(D.cursor==cursor & D.perturbS==0 & D.perturbO==p);
                plot(D.x(indx,:)/10+2,D.y(indx,:)/10,[format{p+2} '.']);hold on;
            end;
            hold off;
            set(gca,'XLim',[-5 3],'YLim',[-0.5 10.5],'Box','off');
            title(figtitle{cursor});xlabel('cm');
            if (cursor==1) ;ylabel('cm');end;
            % plot y-velocity of perturbed hand
            subplot(4,2,cursor+2);
            for p=[-1 0 1]
                indx=find(D.cursor==cursor & D.perturbS==p & D.perturbO==0);
                plot(D.t,D.vy(indx,:),format{p+2} );hold on;
            end;
            hold off;
            set(gca,'XLim',[-50 800],'YLim',[-5 50],'Box','off');
            if (cursor==1)ylabel('y-velocity (cm/s)');end;

            % Plot x-velocity of perturbed hand
            subplot(4,2,cursor+4);
            for p=[-1 0 1]
                indx=find(D.cursor==cursor & D.perturbS==p & D.perturbO==0);
                plot(D.t,D.vx(indx,:),format{p+2});hold on;
            end;
            hold off;
            set(gca,'XLim',[-50 800],'YLim',[-15 15],'Box','off');
            if (cursor==1);ylabel('x-vel (pertub.)'); end;

            % Plot x-velocity of unperturbed hand
            subplot(4,2,cursor+6);
            for p=[-1 0 1]
                indx=find(D.cursor==cursor & D.perturbS==0 & D.perturbO==p);
                plot(D.t,D.vx(indx,:),format{p+2});hold on;
            end;
            hold off;
            set(gca,'XLim',[-50 800],'YLim',[-15 15],'Box','off');
            if (cursor==1);ylabel('x-vel (unpertub.)'); end;

        end;
        set(gcf,'PaperPosition',[2 2 10 25]); % paper position in cm 
        varargout={D};
    case 'simulate_plot'
        shift=4;
        for cursor=1:2
            % Ms=bireach_model('model0');
            % Ms.mix=cursor-1;
            % [x1,x2,x3,M]=bireach_model('predict',Ms);

            [x1,x2,x3,M]=bimoc_simulate('simulate_forcefield',cursor);

            % keyboard;
            i=[1 2 5 6 9 10 13 14];
            x1(i,:)=x1(i,:)*100;
            x2(i,:)=x2(i,:)*100;
            x3(i,:)=x3(i,:)*100;
            % find start of movement (y-vel > 3 cm/s)
            at=findstart(x1(6,:)>3,5);
            t=([1:size(x1,2)]-at)*M.delta_t*1000;

            % Plot x-y plot of the perturbed and unperturbed hand
            subplot(4,2,cursor);
            plot(x1(1,:)+shift,x1(5,:),'k.',x2(1,:)+shift,x2(5,:),'b.',x3(1,:)+shift,x3(5,:),'r.');hold on;
            plot(x1(9,:)-shift,x1(13,:),'k.',x2(9,:)-shift,x2(13,:),'b.',x3(9,:)-shift,x3(13,:),'r.');hold off;
            set(gca,'XLim',[-5 3],'YLim',[-0.5 10.5],'Box','off');
            title(figtitle{cursor});xlabel('cm');
            if (cursor==1)ylabel('cm');end;
            
            % plot y-velocity of perturbed hand
            subplot(4,2,cursor+2);
            plot(t,x1(6,:),'k',t,x2(6,:),'b',t,x3(6,:),'r');
            set(gca,'XLim',[-50 800],'YLim',[-5 50],'Box','off');
            if (cursor==1)ylabel('y-velocity (cm/s)');end;
            
            % Plot x-velocity of perturbed hand
            subplot(4,2,cursor+4);
            plot(t,x1(2,:),'k',t,x2(2,:),'b',t,x3(2,:),'r');
            set(gca,'XLim',[-50 800],'YLim',[-15 15],'Box','off');
            if (cursor==1)ylabel('x-vel (pertub.)');end;

            % Plot x-velocity of unperturbed hand
            subplot(4,2,cursor+6 );
            plot(t,x1(10,:),'k',t,x2(10,:),'b',t,x3(10,:),'r');
            set(gca,'XLim',[-50 800],'YLim',[-15 15],'Box','off');
            if (cursor==1)ylabel('x-vel (unperturb.)');end;
            xlabel('time (ms)'); 

        end;
        set(gcf,'PaperPosition',[2 2 10 25]); % paper position in cm 
        varargout={x1,x2,x3};

end;
