function [Md]=oc_delaysystem(kind,M,varargin);
% delay arbitrary sensory system by discrete delay line 
% Or multi-order exponential low-pass filter 
% Delays only observable states 
% Changes the observation matrix, such that only delayed states are
% observable 
% changes A, and Q accordingly 
% -------------------------------------------------
% Joern Diedrichsen j.diedrichsen@bangor.ac.uk

delay=0.1;      
tau=0.1;
observable=find(any(M.H~=0));       % Find all observable variables 
numstate=length(observable);            
num=1;
vararginoptions(varargin,{'delay','tau','num'},{});
switch (kind) 
    % Delay line is just a transport 
    case 'delay_line'
        N=floor(delay/M.delta_t);
        H=zeros(size(M.H,1),size(M.H,2));
        for i=1:numstate
            H(i,observable(i))=1;
        end;
        Md.A=[M.A zeros(M.ss,numstate*(N-1));...
              H zeros(M.ss,numstate*(N-1));...
              zeros((numstate*(N-1)),M.ss+numstate*(N-1))];
        for j=1:numstate*(N-1)
            Md.A(j+M.ss+numstate,j+M.ss)=1;
        end;
        keyboard;
    case 'exp_filter' 
        H=zeros(size(M.H,1),size(M.H,2));
        for i=1:numstate
            H(i,observable(i))=1;
        end;
        Md=M;
        Md.A=[M.A zeros(M.ss,numstate*num);...
              H./tau*M.delta_t -eye(numstate)./tau*M.delta_t+eye(numstate) zeros(numstate,numstate*(num-1))];
        for i=2:num
            Md.A=[Md.A;zeros(numstate,M.ss+numstate*(i-2)) eye(numstate)./tau*M.delta_t ...
                -eye(numstate)./tau*M.delta_t+eye(numstate) zeros(numstate,numstate*(num-i))];
        end;
        Md.B=[M.B;zeros(numstate*num,size(M.B,2))];
        Md.H=[zeros(M.os,M.ss+numstate*(num-1)) eye(M.os)];
        Md.ss=size(Md.A,1);
        Md.x0=[M.x0;repmat(M.x0(observable),num,1)];
        Md.Q=blockdiag(M.Q,zeros(numstate*num,numstate*num));
end;    